@@ -19,21 +19,21 @@
 
 package org.apache.fory.serializer;
 
+import java.nio.charset.StandardCharsets;
 import java.util.Arrays;
 import java.util.Collection;
 import java.util.List;
-import java.util.Map;
 import java.util.stream.Collectors;
 import org.apache.fory.Fory;
 import org.apache.fory.collection.Tuple2;
 import org.apache.fory.collection.Tuple3;
 import org.apache.fory.exception.ForyException;
+import org.apache.fory.logging.Logger;
+import org.apache.fory.logging.LoggerFactory;
 import org.apache.fory.memory.MemoryBuffer;
-import org.apache.fory.memory.Platform;
 import org.apache.fory.meta.ClassDef;
 import org.apache.fory.reflect.FieldAccessor;
 import org.apache.fory.reflect.ReflectionUtils;
-import org.apache.fory.reflect.TypeRef;
 import org.apache.fory.resolver.ClassInfo;
 import org.apache.fory.resolver.ClassResolver;
 import org.apache.fory.resolver.RefResolver;
@@ -41,9 +41,9 @@ import org.apache.fory.resolver.TypeResolver;
 import org.apache.fory.type.Descriptor;
 import org.apache.fory.type.DescriptorGrouper;
 import org.apache.fory.type.Generics;
-import org.apache.fory.type.TypeUtils;
 import org.apache.fory.type.Types;
-import org.apache.fory.util.Preconditions;
+import org.apache.fory.util.MurmurHash3;
+import org.apache.fory.util.StringUtils;
 import org.apache.fory.util.record.RecordInfo;
 import org.apache.fory.util.record.RecordUtils;
 
@@ -63,6 +63,8 @@ import org.apache.fory.util.record.RecordUtils;
 // TODO(chaokunyang) support generics optimization for {@code SomeClass<T>}
 @SuppressWarnings({"unchecked"})
 public final class ObjectSerializer<T> extends AbstractObjectSerializer<T> {
+  private static final Logger LOG = LoggerFactory.getLogger(ObjectSerializer.class);
+
   private final RecordInfo recordInfo;
   private final FinalTypeField[] finalFields;
 
@@ -96,13 +98,14 @@ public final class ObjectSerializer<T> extends AbstractObjectSerializer<T> {
     Collection<Descriptor> descriptors;
     boolean shareMeta = fory.getConfig().isMetaShareEnabled();
     if (shareMeta) {
-      ClassDef classDef = classResolver.getClassDef(cls, resolveParent);
+      ClassDef classDef = typeResolver.getTypeDef(cls, resolveParent);
       descriptors = classDef.getDescriptors(typeResolver, cls);
     } else {
-      descriptors = fory.getClassResolver().getFieldDescriptors(cls, resolveParent);
+      descriptors = typeResolver.getFieldDescriptors(cls, resolveParent);
     }
-    DescriptorGrouper descriptorGrouper = classResolver.createDescriptorGrouper(descriptors, false);
-    descriptors = descriptorGrouper.getSortedDescriptors();
+    DescriptorGrouper grouper = typeResolver.createDescriptorGrouper(descriptors, false);
+    descriptors = grouper.getSortedDescriptors();
+    System.out.println(descriptors.stream().map(Descriptor::getName).collect(Collectors.toList()));
     if (isRecord) {
       List<String> fieldNames =
           descriptors.stream().map(Descriptor::getName).collect(Collectors.toList());
@@ -111,12 +114,12 @@ public final class ObjectSerializer<T> extends AbstractObjectSerializer<T> {
       recordInfo = null;
     }
     if (fory.checkClassVersion()) {
-      classVersionHash = computeStructHash(fory, descriptors);
+      classVersionHash = computeStructHash(fory, grouper);
     } else {
       classVersionHash = 0;
     }
     Tuple3<Tuple2<FinalTypeField[], boolean[]>, GenericTypeField[], GenericTypeField[]> infos =
-        buildFieldInfos(fory, descriptorGrouper);
+        buildFieldInfos(fory, grouper);
     finalFields = infos.f0.f0;
     isFinal = infos.f0.f1;
     otherFields = infos.f1;
@@ -132,19 +135,15 @@ public final class ObjectSerializer<T> extends AbstractObjectSerializer<T> {
     }
     // write order: primitive,boxed,final,other,collection,map
     writeFinalFields(buffer, value, fory, refResolver, typeResolver);
-    writeOtherFields(buffer, value);
     writeContainerFields(buffer, value, fory, refResolver, typeResolver);
+    writeOtherFields(buffer, value);
   }
 
   private void writeOtherFields(MemoryBuffer buffer, T value) {
     for (GenericTypeField fieldInfo : otherFields) {
       FieldAccessor fieldAccessor = fieldInfo.fieldAccessor;
       Object fieldValue = fieldAccessor.getObject(value);
-      if (fieldInfo.trackingRef) {
-        binding.writeRef(buffer, fieldValue, fieldInfo.classInfoHolder);
-      } else {
-        binding.writeNullable(buffer, fieldValue, fieldInfo.classInfoHolder, fieldInfo.nullable);
-      }
+      writeOtherFieldValue(binding, typeResolver, buffer, fieldInfo, fieldValue);
     }
   }
 
@@ -239,18 +238,32 @@ public final class ObjectSerializer<T> extends AbstractObjectSerializer<T> {
     }
   }
 
+  static void writeOtherFieldValue(
+      SerializationBinding binding,
+      TypeResolver typeResolver,
+      MemoryBuffer buffer,
+      GenericTypeField fieldInfo,
+      Object fieldValue) {
+    if (fieldValue == null) {
+      buffer.writeByte(Fory.NULL_FLAG);
+    } else if (fieldValue.getClass().isEnum()) {
+      buffer.writeByte(Fory.NOT_NULL_VALUE_FLAG);
+      fieldInfo.genericType.getSerializer(typeResolver).write(buffer, fieldValue);
+    } else if (fieldInfo.trackingRef) {
+      binding.writeRef(buffer, fieldValue, fieldInfo.classInfoHolder);
+    } else {
+      binding.writeNullable(buffer, fieldValue, fieldInfo.classInfoHolder, fieldInfo.nullable);
+    }
+  }
+
   @Override
   public T read(MemoryBuffer buffer) {
     if (isRecord) {
       Object[] fields = readFields(buffer);
       fields = RecordUtils.remapping(recordInfo, fields);
-      try {
-        T obj = (T) constructor.invokeWithArguments(fields);
-        Arrays.fill(recordInfo.getRecordComponents(), null);
-        return obj;
-      } catch (Throwable e) {
-        Platform.throwException(e);
-      }
+      T obj = (T) objectCreator.newInstanceWithArguments(fields);
+      Arrays.fill(recordInfo.getRecordComponents(), null);
+      return obj;
     }
     T obj = newBean();
     refResolver.reference(obj);
@@ -290,15 +303,15 @@ public final class ObjectSerializer<T> extends AbstractObjectSerializer<T> {
         fieldValues[counter++] = fieldValue;
       }
     }
-    for (GenericTypeField fieldInfo : otherFields) {
-      Object fieldValue = readOtherFieldValue(binding, fieldInfo, buffer);
-      fieldValues[counter++] = fieldValue;
-    }
     Generics generics = fory.getGenerics();
     for (GenericTypeField fieldInfo : containerFields) {
       Object fieldValue = readContainerFieldValue(binding, generics, fieldInfo, buffer);
       fieldValues[counter++] = fieldValue;
     }
+    for (GenericTypeField fieldInfo : otherFields) {
+      Object fieldValue = readOtherFieldValue(binding, fieldInfo, buffer);
+      fieldValues[counter++] = fieldValue;
+    }
     return fieldValues;
   }
 
@@ -329,64 +342,82 @@ public final class ObjectSerializer<T> extends AbstractObjectSerializer<T> {
         fieldAccessor.putObject(obj, fieldValue);
       }
     }
-    for (GenericTypeField fieldInfo : otherFields) {
-      Object fieldValue = readOtherFieldValue(binding, fieldInfo, buffer);
-      FieldAccessor fieldAccessor = fieldInfo.fieldAccessor;
-      fieldAccessor.putObject(obj, fieldValue);
-    }
     Generics generics = fory.getGenerics();
     for (GenericTypeField fieldInfo : containerFields) {
       Object fieldValue = readContainerFieldValue(binding, generics, fieldInfo, buffer);
       FieldAccessor fieldAccessor = fieldInfo.fieldAccessor;
       fieldAccessor.putObject(obj, fieldValue);
     }
+    for (GenericTypeField fieldInfo : otherFields) {
+      Object fieldValue = readOtherFieldValue(binding, fieldInfo, buffer);
+      FieldAccessor fieldAccessor = fieldInfo.fieldAccessor;
+      fieldAccessor.putObject(obj, fieldValue);
+    }
     return obj;
   }
 
-  public static int computeStructHash(Fory fory, Collection<Descriptor> descriptors) {
-    int hash = 17;
-    for (Descriptor descriptor : descriptors) {
-      hash = computeFieldHash(hash, fory, descriptor.getTypeRef());
+  public static int computeStructHash(Fory fory, DescriptorGrouper grouper) {
+    StringBuilder builder = new StringBuilder();
+    List<Descriptor> sorted = grouper.getSortedDescriptors();
+    for (Descriptor descriptor : sorted) {
+      Class<?> rawType = descriptor.getTypeRef().getRawType();
+      int typeId = getTypeId(fory, rawType);
+      String underscore = StringUtils.lowerCamelToLowerUnderscore(descriptor.getName());
+      char nullable = rawType.isPrimitive() ? '0' : '1';
+      builder
+          .append(underscore)
+          .append(',')
+          .append(typeId)
+          .append(',')
+          .append(nullable)
+          .append(';');
+    }
+
+    String fingerprint = builder.toString();
+    byte[] bytes = fingerprint.getBytes(StandardCharsets.UTF_8);
+    long hashLong = MurmurHash3.murmurhash3_x64_128(bytes, 0, bytes.length, 47)[0];
+    int hash = (int) (hashLong & 0xffffffffL);
+    if (fory.getConfig().isForyDebugOutputEnabled()) {
+      String className =
+          sorted.isEmpty() ? "<unknown>" : String.valueOf(sorted.get(0).getDeclaringClass());
+      LOG.info(
+          "[fory-debug] struct "
+              + className
+              + " version fingerprint=\""
+              + fingerprint
+              + "\" version hash="
+              + hash);
     }
-    Preconditions.checkState(hash != 0);
     return hash;
   }
 
-  private static int computeFieldHash(int hash, Fory fory, TypeRef<?> typeRef) {
-    int id;
-    if (typeRef.isSubtypeOf(List.class)) {
-      // TODO(chaokunyang) add list element type into schema hash
-      id = Types.LIST;
-    } else if (typeRef.isSubtypeOf(Map.class)) {
-      // TODO(chaokunyang) add map key&value type into schema hash
-      id = Types.MAP;
+  private static int getTypeId(Fory fory, Class<?> cls) {
+    TypeResolver resolver = fory._getTypeResolver();
+    if (resolver.isSet(cls)) {
+      return Types.SET;
+    } else if (resolver.isCollection(cls)) {
+      return Types.LIST;
+    } else if (resolver.isMap(cls)) {
+      return Types.MAP;
     } else {
-      try {
-        TypeResolver resolver =
-            fory.isCrossLanguage() ? fory.getXtypeResolver() : fory.getClassResolver();
-        Class<?> cls = typeRef.getRawType();
-        if (ReflectionUtils.isAbstract(cls) || cls.isInterface()) {
-          id = 0;
-        } else {
-          ClassInfo classInfo = resolver.getClassInfo(typeRef.getRawType());
-          int xtypeId = classInfo.getXtypeId();
-          if (Types.isStructType((byte) xtypeId)) {
-            id =
-                TypeUtils.computeStringHash(
-                    classInfo.decodeNamespace() + classInfo.decodeTypeName());
-          } else {
-            id = Math.abs(xtypeId);
-          }
-        }
-      } catch (Exception e) {
-        id = 0;
+      if (ReflectionUtils.isAbstract(cls) || cls.isInterface() || cls.isEnum()) {
+        return Types.UNKNOWN;
       }
+      ClassInfo classInfo = resolver.getClassInfo(cls, false);
+      if (classInfo == null) {
+        return Types.UNKNOWN;
+      }
+      int typeId;
+      if (fory.isCrossLanguage()) {
+        typeId = classInfo.getXtypeId();
+        if (Types.isUserDefinedType((byte) typeId)) {
+          return Types.UNKNOWN;
+        }
+      } else {
+        typeId = classInfo.getClassId();
+      }
+      return typeId;
     }
-    long newHash = ((long) hash) * 31 + id;
-    while (newHash >= Integer.MAX_VALUE) {
-      newHash /= 7;
-    }
-    return (int) newHash;
   }
 
   public static void checkClassVersion(Fory fory, int readHash, int classVersionHash) {
