@@ -783,6 +783,55 @@ public class CollectionSerializers {
     }
   }
 
+  public abstract static class XlangCollectionDefaultSerializer extends CollectionLikeSerializer {
+
+    public XlangCollectionDefaultSerializer(Fory fory, Class cls) {
+      super(fory, cls);
+    }
+
+    @Override
+    public Collection onCollectionWrite(MemoryBuffer buffer, Object value) {
+      Collection v = (Collection) value;
+      buffer.writeVarUint32Small7(v.size());
+      return v;
+    }
+
+    @Override
+    public Object onCollectionRead(Collection collection) {
+      return collection;
+    }
+  }
+
+  public static class XlangListDefaultSerializer extends XlangCollectionDefaultSerializer {
+    public XlangListDefaultSerializer(Fory fory, Class cls) {
+      super(fory, cls);
+    }
+
+    @Override
+    public List newCollection(MemoryBuffer buffer) {
+      int numElements = buffer.readVarUint32Small7();
+      setNumElements(numElements);
+      ArrayList list = new ArrayList(numElements);
+      fory.getRefResolver().reference(list);
+      return list;
+    }
+  }
+
+  public static class XlangSetDefaultSerializer extends XlangCollectionDefaultSerializer {
+    public XlangSetDefaultSerializer(Fory fory, Class cls) {
+      super(fory, cls);
+    }
+
+    @Override
+    public Set newCollection(MemoryBuffer buffer) {
+      int numElements = buffer.readVarUint32Small7();
+      setNumElements(numElements);
+      HashSet set = new HashSet(numElements);
+      fory.getRefResolver().reference(set);
+      return set;
+    }
+  }
+
   // TODO add JDK11:JdkImmutableListSerializer,JdkImmutableMapSerializer,JdkImmutableSetSerializer
   //  by jit codegen those constructor for compiling in jdk8.
   // TODO Support ArraySubListSerializer, SubListSerializer
