@@ -97,11 +97,6 @@ public abstract class ForyTestBase {
     return new Object[][] {{false}, {true}};
   }
 
-  @DataProvider
-  public static Object[][] compatible() {
-    return new Object[][] {{false}, {true}};
-  }
-
   @DataProvider
   public static Object[][] trackingRefFory() {
     return new Object[][] {
@@ -136,6 +131,11 @@ public abstract class ForyTestBase {
     return new Object[][] {{false}, {true}};
   }
 
+  @DataProvider
+  public static Object[][] compatible() {
+    return new Object[][] {{false}, {true}};
+  }
+
   @DataProvider
   public static Object[][] compressNumber() {
     return new Object[][] {{false}, {true}};
@@ -181,6 +181,11 @@ public abstract class ForyTestBase {
     };
   }
 
+  @DataProvider
+  public static Object[][] language() {
+    return new Object[][] {{Language.JAVA}, {Language.XLANG}};
+  }
+
   @DataProvider(name = "javaFory")
   public static Object[][] javaForyConfig() {
     return new Object[][] {
