@@ -211,7 +211,7 @@ def test_record_batch(data_file_path):
         # debug_print(f"batch[0] {batch[0]}")
 
         encoder = pyfory.create_row_encoder(create_foo_schema())
-        writer = pyfory.ArrowWriter(create_foo_schema())
+        writer = pyfory.format.ArrowWriter(create_foo_schema())
         num_rows = 128
         for i in range(num_rows):
             foo = create_foo()
@@ -275,7 +275,7 @@ def test_cross_language_serializer(data_file_path):
     with open(data_file_path, "rb") as f:
         data_bytes = f.read()
         buffer = pyfory.Buffer(data_bytes)
-        fory = pyfory.Fory(language=pyfory.Language.XLANG, ref_tracking=True)
+        fory = pyfory.Fory(xlang=True, ref=True)
         objects = []
         assert _deserialize_and_append(fory, buffer, objects) is True
         assert _deserialize_and_append(fory, buffer, objects) is False
@@ -350,7 +350,7 @@ def test_cross_language_reference(data_file_path):
     with open(data_file_path, "rb") as f:
         data_bytes = f.read()
         buffer = pyfory.Buffer(data_bytes)
-        fory = pyfory.Fory(language=pyfory.Language.XLANG, ref_tracking=True)
+        fory = pyfory.Fory(xlang=True, ref=True)
         new_list = fory.deserialize(buffer)
         assert new_list[0] is new_list
         new_map = new_list[1]
@@ -376,7 +376,7 @@ def test_serialize_arrow_in_band(data_file_path):
         table = pa.Table.from_batches([batch] * 2)
         data_bytes = f.read()
         buffer = pyfory.Buffer(data_bytes)
-        fory = pyfory.Fory(language=pyfory.Language.XLANG, ref_tracking=True)
+        fory = pyfory.Fory(xlang=True, ref=True)
         new_batch = fory.deserialize(buffer)
         assert new_batch == batch
         new_table = fory.deserialize(buffer)
@@ -398,12 +398,12 @@ def test_serialize_arrow_out_of_band(int_band_file, out_of_band_file):
         out_of_band_buffer.slice(8, len1),
         out_of_band_buffer.slice(8 + len1, len2),
     ]
-    fory = pyfory.Fory(language=pyfory.Language.XLANG, ref_tracking=True)
+    fory = pyfory.Fory(xlang=True, ref=True)
     objects = fory.deserialize(in_band_buffer, buffers=buffers)
     assert objects == [batch, table]
     buffer_objects = []
     in_band_buffer = fory.serialize([batch, table], buffer_callback=buffer_objects.append)
-    buffers = [o.to_buffer() for o in buffer_objects]
+    buffers = [o.getbuffer() for o in buffer_objects]
     with open(int_band_file, "wb+") as f:
         f.write(in_band_buffer)
     with open(out_of_band_file, "wb+") as f:
@@ -429,25 +429,25 @@ class ComplexObject1:
     f1: Any = None
     f2: str = None
     f3: List[str] = None
-    f4: Dict[pyfory.Int8Type, pyfory.Int32Type] = None
-    f5: pyfory.Int8Type = None
-    f6: pyfory.Int16Type = None
-    f7: pyfory.Int32Type = None
-    f8: pyfory.Int64Type = None
-    f9: pyfory.Float32Type = None
-    f10: pyfory.Float64Type = None
-    f11: pyfory.Int16ArrayType = None
-    f12: List[pyfory.Int16Type] = None
+    f4: Dict[pyfory.int8, pyfory.int32] = None
+    f5: pyfory.int8 = None
+    f6: pyfory.int16 = None
+    f7: pyfory.int32 = None
+    f8: pyfory.int64 = None
+    f9: pyfory.float32 = None
+    f10: pyfory.float64 = None
+    f11: pyfory.int16_array = None
+    f12: List[pyfory.int16] = None
 
 
 @dataclass
 class ComplexObject2:
     f1: Any
-    f2: Dict[pyfory.Int8Type, pyfory.Int32Type]
+    f2: Dict[pyfory.int8, pyfory.int32]
 
 
 def test_serialize_simple_struct_local():
-    fory = pyfory.Fory(language=pyfory.Language.XLANG, ref_tracking=True)
+    fory = pyfory.Fory(xlang=True, ref=True)
     fory.register_type(ComplexObject2, namespace="test", typename="ComplexObject2")
     obj = ComplexObject2(f1=True, f2={-1: 2})
     new_buf = fory.serialize(obj)
@@ -456,12 +456,22 @@ def test_serialize_simple_struct_local():
 
 @cross_language_test
 def test_serialize_simple_struct(data_file_path):
-    fory = pyfory.Fory(language=pyfory.Language.XLANG, ref_tracking=True)
+    compatible = "compatible" in data_file_path
+    fory = pyfory.Fory(xlang=True, ref=True, compatible=compatible)
     fory.register_type(ComplexObject2, namespace="test", typename="ComplexObject2")
     obj = ComplexObject2(f1=True, f2={-1: 2})
     struct_round_back(data_file_path, fory, obj)
 
 
+@cross_language_test
+def test_register_by_id(data_file_path):
+    compatible = "compatible" in data_file_path
+    fory = pyfory.Fory(xlang=True, ref=True, compatible=compatible)
+    fory.register_type(ComplexObject2, type_id=100)
+    obj = ComplexObject2(f1=True, f2={-1: 2})
+    struct_round_back(data_file_path, fory, obj)
+
+
 class SomeClass:
     f1: "SomeClass"
     f2: Dict[str, str]
@@ -469,7 +479,7 @@ class SomeClass:
 
 
 def test_custom_class_roundtrip():
-    fory = pyfory.Fory(ref_tracking=True)
+    fory = pyfory.Fory(ref=True)
     fory.register_type(SomeClass, typename="example.SomeClass")
     obj1 = SomeClass()
     obj1.f2 = {"k1": "v1", "k2": "v2"}
@@ -505,31 +515,48 @@ class EnumFieldStruct:
 
 @cross_language_test
 def test_enum_field(data_file_path):
-    fory = pyfory.Fory(language=pyfory.Language.XLANG, ref_tracking=False)
+    compatible = "compatible" in data_file_path
+    fory = pyfory.Fory(xlang=True, ref=False, compatible=compatible)
     fory.register_type(EnumTestClass, namespace="test", typename="EnumTestClass")
     fory.register_type(EnumFieldStruct, namespace="test", typename="EnumFieldStruct")
     obj = EnumFieldStruct(f1=EnumTestClass.FOO, f2=EnumTestClass.BAR, f3="abc")
     struct_round_back(data_file_path, fory, obj)
 
 
+@cross_language_test
+def test_enum_field_register_by_id(data_file_path):
+    compatible = "compatible" in data_file_path
+    fory = pyfory.Fory(xlang=True, ref=False, compatible=compatible)
+    fory.register_type(EnumTestClass, type_id=1)
+    fory.register_type(EnumFieldStruct, type_id=2)
+    obj = EnumFieldStruct(f1=EnumTestClass.FOO, f2=EnumTestClass.BAR, f3="abc")
+    struct_round_back(data_file_path, fory, obj)
+
+
 @cross_language_test
 def test_struct_hash(data_file_path):
     with open(data_file_path, "rb") as f:
         data_bytes = f.read()
     debug_print(f"len {len(data_bytes)}")
     read_hash = pyfory.Buffer(data_bytes).read_int32()
-    fory = pyfory.Fory(language=pyfory.Language.XLANG, ref_tracking=True)
+    fory = pyfory.Fory(xlang=True, ref=True)
     fory.register_type(ComplexObject1, typename="ComplexObject1")
-    serializer = fory.type_resolver.get_serializer(ComplexObject1)
-    from pyfory._struct import _get_hash
+    serializer = fory.type_resolver.get_serializer(ComplexObject1)._replace()
+    from pyfory._struct import compute_struct_meta
 
-    v = _get_hash(fory, serializer._field_names, serializer._type_hints)
+    v = compute_struct_meta(
+        fory.type_resolver,
+        serializer._field_names,
+        serializer._serializers,
+        serializer._nullable_fields,
+    )[0]
     assert read_hash == v, (read_hash, v)
 
 
 @cross_language_test
 def test_serialize_complex_struct(data_file_path):
-    fory = pyfory.Fory(language=pyfory.Language.XLANG, ref_tracking=True)
+    compatible = "compatible" in data_file_path
+    fory = pyfory.Fory(xlang=True, ref=True, compatible=compatible)
     fory.register_type(ComplexObject1, namespace="test", typename="ComplexObject1")
     fory.register_type(ComplexObject2, namespace="test", typename="ComplexObject2")
 
@@ -575,16 +602,16 @@ class ComplexObject1Serializer(pyfory.serializer.Serializer):
         return self.xread(buffer)
 
     def xwrite(self, buffer, value):
-        self.fory.xserialize_ref(buffer, value.f1)
-        self.fory.xserialize_ref(buffer, value.f2)
-        self.fory.xserialize_ref(buffer, value.f3)
+        self.fory.xwrite_ref(buffer, value.f1)
+        self.fory.xwrite_ref(buffer, value.f2)
+        self.fory.xwrite_ref(buffer, value.f3)
 
     def xread(self, buffer):
         obj = ComplexObject1(*([None] * len(typing.get_type_hints(ComplexObject1).keys())))
         self.fory.ref_resolver.reference(obj)
-        obj.f1 = self.fory.xdeserialize_ref(buffer)
-        obj.f2 = self.fory.xdeserialize_ref(buffer)
-        obj.f3 = self.fory.xdeserialize_ref(buffer)
+        obj.f1 = self.fory.xread_ref(buffer)
+        obj.f2 = self.fory.xread_ref(buffer)
+        obj.f3 = self.fory.xread_ref(buffer)
         return obj
 
 
@@ -594,7 +621,7 @@ def test_register_serializer(data_file_path):
         data_bytes = f.read()
     buffer = pyfory.Buffer(data_bytes)
 
-    fory = pyfory.Fory(language=pyfory.Language.XLANG, ref_tracking=True)
+    fory = pyfory.Fory(xlang=True, ref=True)
     fory.register_type(
         ComplexObject1,
         typename="test.ComplexObject1",
@@ -611,7 +638,7 @@ def test_register_serializer(data_file_path):
     new_buf = pyfory.Buffer.allocate(32)
     fory.serialize(new_obj, buffer=new_buf)
     bytes1 = fory.serialize(new_obj)
-    assert len(bytes1) == len(data_bytes)
+    assert len(bytes1) == len(data_bytes), (bytes1, data_bytes)
     # header can be different to embed writer info like language
     assert bytes1[8:] == data_bytes[8:]
     assert fory.deserialize(fory.serialize(new_obj)) == new_obj, new_obj
@@ -626,7 +653,7 @@ def test_oob_buffer(in_band_file_path, out_of_band_file_path):
         in_band_bytes = f.read()
     with open(out_of_band_file_path, "rb") as f:
         out_of_band_buffer = pyfory.Buffer(f.read())
-    fory = pyfory.Fory(language=pyfory.Language.XLANG, ref_tracking=True)
+    fory = pyfory.Fory(xlang=True, ref=True)
     n_buffers = out_of_band_buffer.read_int32()
     buffers = []
     for i in range(n_buffers):
@@ -654,8 +681,8 @@ def test_oob_buffer(in_band_file_path, out_of_band_file_path):
     # in_band_bytes size may be different because it may contain language-specific meta.
     debug_print(f"{len(serialized), len(in_band_bytes)}")
     debug_print(f"deserialized from other language {new_obj}")
-    debug_print(f"deserialized from python {fory.deserialize(serialized, [o.to_buffer() for o in buffer_objects])}")
-    fory.deserialize(serialized, [o.to_buffer() for o in buffer_objects])
+    debug_print(f"deserialized from python {fory.deserialize(serialized, [o.getbuffer() for o in buffer_objects])}")
+    fory.deserialize(serialized, [o.getbuffer() for o in buffer_objects])
     with open(in_band_file_path, "wb+") as f:
         f.write(serialized)
     out_of_band_buffer.write_int32(len(buffer_objects))
@@ -666,6 +693,267 @@ def test_oob_buffer(in_band_file_path, out_of_band_file_path):
         f.write(out_of_band_buffer.to_bytes(0, out_of_band_buffer.writer_index))
 
 
+@cross_language_test
+def test_cross_language_meta_share(data_file_path):
+    """Test cross-language meta sharing with ComplexObject2."""
+    fory = pyfory.Fory(xlang=True, compatible=True, ref=True)
+
+    @dataclass
+    class ComplexObject2:
+        f1: Any
+        f2: Dict[pyfory.int8, pyfory.int32]
+
+    fory.register_type(ComplexObject2, namespace="test", typename="ComplexObject2")
+
+    with open(data_file_path, "rb") as f:
+        data_bytes = f.read()
+
+    debug_print(f"Reading data of length {len(data_bytes)} from {data_file_path}")
+
+    # Deserialize Java-generated data with meta share
+    obj = fory.deserialize(data_bytes)
+    debug_print(f"Deserialized object: {obj}")
+
+    # Verify the object structure
+    assert obj.f1 is True  # Boolean value
+    assert isinstance(obj.f2, dict)
+    assert obj.f2[-1] == 2  # Dict[Int8, Int32] with key=-1, value=2
+
+    # Serialize back with meta share
+    new_serialized = fory.serialize(obj)
+    debug_print(f"Re-serialized data length: {len(new_serialized)}")
+
+    # Verify round-trip
+    round_trip_obj = fory.deserialize(new_serialized)
+    assert round_trip_obj == obj
+
+    # Write back for Java to verify
+    with open(data_file_path, "wb") as f:
+        f.write(new_serialized)
+
+
+@cross_language_test
+def test_cross_language_meta_share_complex(data_file_path):
+    """Test cross-language meta sharing with complex nested objects."""
+    fory = pyfory.Fory(xlang=True, compatible=True, ref=True)
+
+    @dataclass
+    class ComplexObject2:
+        f1: Any
+        f2: Dict[pyfory.int8, pyfory.int32]
+
+    @dataclass
+    class ComplexObject1:
+        f1: Any
+        f2: str
+        f3: List[str]
+        f4: Dict[pyfory.int8, pyfory.int32]
+        f5: pyfory.int8
+        f6: pyfory.int16
+        f7: pyfory.int32
+        f8: pyfory.int64
+        f9: pyfory.float32
+        f10: pyfory.float64
+        f11: pyfory.int16_array
+        f12: List[pyfory.int16]
+
+    fory.register_type(ComplexObject1, namespace="test", typename="ComplexObject1")
+    fory.register_type(ComplexObject2, namespace="test", typename="ComplexObject2")
+
+    with open(data_file_path, "rb") as f:
+        data_bytes = f.read()
+
+    debug_print(f"Reading complex data of length {len(data_bytes)} from {data_file_path}")
+
+    # Deserialize Java-generated complex object with meta share
+    obj = fory.deserialize(data_bytes)
+    debug_print(f"Deserialized complex object: {obj}")
+
+    # Verify the nested object structure
+    assert hasattr(obj, "f1") and hasattr(obj.f1, "f1") and hasattr(obj.f1, "f2")
+    assert obj.f1.f1 is True
+    assert isinstance(obj.f1.f2, dict)
+    assert obj.f2 == "meta_share_test"
+    assert obj.f3 == ["compatible", "mode"]
+
+    # Serialize back with meta share
+    new_serialized = fory.serialize(obj)
+    debug_print(f"Re-serialized complex data length: {len(new_serialized)}")
+
+    # Verify round-trip
+    round_trip_obj = fory.deserialize(new_serialized)
+    assert round_trip_obj == obj
+
+    # Write back for Java to verify
+    with open(data_file_path, "wb") as f:
+        f.write(new_serialized)
+
+
+@cross_language_test
+def test_schema_evolution(data_file_path):
+    """Test schema evolution compatibility."""
+    fory = pyfory.Fory(xlang=True, compatible=True, ref=True)
+
+    # Same V1 class reading V1 data - should work perfectly
+    @dataclass
+    class CompatTestV1:
+        name: str
+        age: pyfory.int32  # Use specific fory type to match Java Integer
+
+    fory.register_type(CompatTestV1, namespace="test", typename="CompatTest")
+
+    with open(data_file_path, "rb") as f:
+        data_bytes = f.read()
+
+    debug_print(f"Reading schema evolution data of length {len(data_bytes)} from {data_file_path}")
+
+    # Deserialize V1 data into V1 object
+    obj = fory.deserialize(data_bytes)
+    debug_print(f"Deserialized V1 object: {obj}")
+
+    # Verify the object structure
+    assert obj.name == "Schema Evolution Test"
+    assert obj.age == 42
+
+    # Test round trip
+    round_trip_obj = fory.deserialize(fory.serialize(obj))
+    assert round_trip_obj == obj
+
+    # Serialize back
+    new_serialized = fory.serialize(obj)
+    debug_print(f"Re-serialized data length: {len(new_serialized)}")
+    assert fory.deserialize(new_serialized) == obj
+
+    # Write back for Java to verify
+    with open(data_file_path, "wb") as f:
+        f.write(new_serialized)
+
+
+@cross_language_test
+def test_backward_compatibility(data_file_path):
+    """Test backward compatibility - old version reading new data."""
+    fory = pyfory.Fory(xlang=True, compatible=True, ref=True)
+
+    # Version 1 class (original) reading Version 2 data (should ignore unknown fields)
+    @dataclass
+    class CompatTestV1:
+        name: str
+        age: pyfory.int32
+
+    fory.register_type(CompatTestV1, namespace="test", typename="CompatTest")
+
+    with open(data_file_path, "rb") as f:
+        data_bytes = f.read()
+
+    debug_print(f"Reading V2 data of length {len(data_bytes)} from {data_file_path}")
+
+    # Deserialize V2 data into V1 object (ignoring email field)
+    obj = fory.deserialize(data_bytes)
+    debug_print(f"Deserialized V1 object from V2 data: {obj}")
+
+    # Verify the object structure - only V1 fields should be present
+    assert obj.name == "Bob"
+    assert obj.age == 30
+    # email field should be ignored/not present
+
+    # Serialize back with V1 schema
+    new_serialized = fory.serialize(obj)
+    debug_print(f"Re-serialized V1 data length: {len(new_serialized)}")
+
+    # Write back for Java to verify
+    with open(data_file_path, "wb") as f:
+        f.write(new_serialized)
+
+
+@cross_language_test
+def test_field_reordering_compatibility(data_file_path):
+    """Test field reordering compatibility in metashare mode."""
+    fory = pyfory.Fory(xlang=True, ref=True, compatible=True)
+
+    # Version 3 class with reordered fields matching Java CompatTestV3
+    @dataclass
+    class CompatTestV3:
+        age: pyfory.int32  # Reordered (was second in V1)
+        name: str  # Reordered (was first in V1)
+        email: str
+        active: bool  # New field
+
+    fory.register_type(CompatTestV3, namespace="test", typename="CompatTest")
+
+    with open(data_file_path, "rb") as f:
+        data_bytes = f.read()
+
+    debug_print(f"Reading V3 reordered data of length {len(data_bytes)} from {data_file_path}")
+
+    # Deserialize V3 data into V3 object
+    obj = fory.deserialize(data_bytes)
+    debug_print(f"Deserialized V3 object from V3 reordered data: {obj}")
+
+    # Verify the object structure
+    assert obj.age == 35
+    assert obj.name == "Charlie"
+    assert obj.email == "charlie@example.com"
+    assert obj.active
+
+    # Serialize back with V3 schema
+    new_serialized = fory.serialize(obj)
+    debug_print(f"Re-serialized V3 data length: {len(new_serialized)}")
+
+    # Write back for Java to verify
+    with open(data_file_path, "wb") as f:
+        f.write(new_serialized)
+
+
+@cross_language_test
+def test_cross_version_compatibility(data_file_path):
+    """Test mixed version compatibility."""
+    fory = pyfory.Fory(xlang=True, compatible=True, ref=True)
+
+    @dataclass
+    class CompatTestV1:
+        name: str
+        age: pyfory.int32
+
+    @dataclass
+    class CompatTestV2:
+        name: str
+        age: pyfory.int32
+        email: str = "default@example.com"
+
+    @dataclass
+    class CompatContainer:
+        oldObject: CompatTestV1
+        newObject: CompatTestV2
+
+    fory.register_type(CompatContainer, namespace="test", typename="CompatContainer")
+    fory.register_type(CompatTestV1, namespace="test", typename="CompatTestV1")
+    fory.register_type(CompatTestV2, namespace="test", typename="CompatTestV2")
+
+    with open(data_file_path, "rb") as f:
+        data_bytes = f.read()
+
+    debug_print(f"Reading mixed version data of length {len(data_bytes)} from {data_file_path}")
+
+    # Deserialize mixed version container
+    obj = fory.deserialize(data_bytes)
+    debug_print(f"Deserialized mixed version container: {obj}")
+
+    # Verify the nested objects
+    assert obj.oldObject.name == "Old Format", obj.oldObject.name
+    assert obj.oldObject.age == 20, obj.oldObject.age
+    assert obj.newObject.name == "New Format", obj.newObject.name
+    assert obj.newObject.age == 25, obj.newObject.age
+    assert obj.newObject.email == "new@example.com", obj.newObject.email
+
+    # Serialize back
+    new_serialized = fory.serialize(obj)
+    debug_print(f"Re-serialized mixed version data length: {len(new_serialized)}")
+
+    # Write back for Java to verify
+    with open(data_file_path, "wb") as f:
+        f.write(new_serialized)
+
+
 if __name__ == "__main__":
     """This file will be executed by CrossLanguageTest.java in fory-core/fory-format module and
     fory_xlang_test.go in go/fory module
@@ -679,7 +967,7 @@ if __name__ == "__main__":
     try:
         args = sys.argv[1:]
         assert len(args) > 0
-        func = getattr(sys.modules[__name__], args[0])
+        func = getattr(sys.modules[__name__], args[0].replace("_compatible", ""))
         if not func:
             raise Exception("Unknown args {}".format(args))
         func(*args[1:])
