@@ -15,17 +15,27 @@
 // specific language governing permissions and limitations
 // under the License.
 
-use anyhow::anyhow;
-
 use crate::ensure;
 use crate::error::Error;
 use crate::meta::string_util;
+use std::sync::OnceLock;
 
 // equal to "std::i16::MAX"
 const SHORT_MAX_VALUE: usize = 32767;
+// const HEADER_MASK:i64 = 0xff;
 
-#[derive(Debug, PartialEq)]
+pub static NAMESPACE_ENCODER: MetaStringEncoder = MetaStringEncoder::new('.', '_');
+pub static TYPE_NAME_ENCODER: MetaStringEncoder = MetaStringEncoder::new('$', '_');
+pub static FIELD_NAME_ENCODER: MetaStringEncoder = MetaStringEncoder::new('$', '_');
+
+pub static NAMESPACE_DECODER: MetaStringDecoder = MetaStringDecoder::new('.', '_');
+pub static FIELD_NAME_DECODER: MetaStringDecoder = MetaStringDecoder::new('$', '_');
+pub static TYPE_NAME_DECODER: MetaStringDecoder = MetaStringDecoder::new('$', '_');
+
+#[derive(Debug, PartialEq, Hash, Eq, Clone, Copy, Default)]
+#[repr(i16)]
 pub enum Encoding {
+    #[default]
     Utf8 = 0x00,
     LowerSpecial = 0x01,
     LowerUpperDigitSpecial = 0x02,
@@ -33,20 +43,45 @@ pub enum Encoding {
     AllToLowerSpecial = 0x04,
 }
 
-#[derive(Debug, PartialEq)]
+#[derive(Debug, Clone, Default)]
 pub struct MetaString {
     pub original: String,
     pub encoding: Encoding,
     pub bytes: Vec<u8>,
     pub strip_last_char: bool,
+    pub special_char1: char,
+    pub special_char2: char,
 }
 
+impl PartialEq for MetaString {
+    fn eq(&self, other: &Self) -> bool {
+        self.bytes == other.bytes
+    }
+}
+
+impl Eq for MetaString {}
+
+impl std::hash::Hash for MetaString {
+    fn hash<H: std::hash::Hasher>(&self, state: &mut H) {
+        self.bytes.hash(state);
+    }
+}
+
+static EMPTY: OnceLock<MetaString> = OnceLock::new();
+
 impl MetaString {
-    pub fn new(original: String, encoding: Encoding, bytes: Vec<u8>) -> Result<Self, Error> {
+    pub fn new(
+        original: String,
+        encoding: Encoding,
+        bytes: Vec<u8>,
+        special_char1: char,
+        special_char2: char,
+    ) -> Result<Self, Error> {
         let mut strip_last_char = false;
         if encoding != Encoding::Utf8 {
-            ensure!(!bytes.is_empty(), anyhow!("Encoded data cannot be empty"));
-
+            if bytes.is_empty() {
+                return Err(Error::encode_error("Encoded data cannot be empty"));
+            }
             strip_last_char = (bytes[0] & 0x80) != 0;
         }
         Ok(MetaString {
@@ -54,22 +89,38 @@ impl MetaString {
             encoding,
             bytes,
             strip_last_char,
+            special_char1,
+            special_char2,
+        })
+    }
+
+    pub fn write_to(&self, writer: &mut crate::buffer::Writer) {
+        writer.write_varuint32(self.bytes.len() as u32);
+        writer.write_bytes(&self.bytes);
+    }
+
+    pub fn get_empty() -> &'static MetaString {
+        EMPTY.get_or_init(|| MetaString {
+            original: "".to_string(),
+            encoding: Encoding::default(),
+            bytes: vec![],
+            strip_last_char: false,
+            special_char1: '\0',
+            special_char2: '\0',
         })
     }
 }
 
-pub struct MetaStringDecoder {}
-impl Default for MetaStringDecoder {
-    fn default() -> Self {
-        Self::new()
-    }
+#[derive(Clone)]
+pub struct MetaStringDecoder {
+    pub special_char1: char,
+    pub special_char2: char,
 }
 
-pub struct MetaStringEncoder {}
-impl Default for MetaStringEncoder {
-    fn default() -> Self {
-        Self::new()
-    }
+#[derive(Clone)]
+pub struct MetaStringEncoder {
+    pub special_char1: char,
+    pub special_char2: char,
 }
 
 #[derive(Debug)]
@@ -81,49 +132,94 @@ struct StringStatistics {
 }
 
 impl MetaStringEncoder {
-    pub fn new() -> Self {
-        MetaStringEncoder {}
+    pub const fn new(special_char1: char, special_char2: char) -> Self {
+        Self {
+            special_char1,
+            special_char2,
+        }
     }
 
     fn is_latin(&self, s: &str) -> bool {
         string_util::is_latin(s)
     }
 
-    pub fn encode(&self, input: &str) -> Result<MetaString, Error> {
+    fn _encode(&self, input: &str) -> Result<Option<MetaString>, Error> {
         if input.is_empty() {
-            return MetaString::new(input.to_string(), Encoding::Utf8, vec![]);
+            return Ok(Some(MetaString::new(
+                input.to_string(),
+                Encoding::Utf8,
+                vec![],
+                self.special_char1,
+                self.special_char2,
+            )?));
         }
+
         ensure!(
             input.len() < SHORT_MAX_VALUE,
-            anyhow!(
+            Error::encode_error(format!(
                 "Meta string is too long, max:{SHORT_MAX_VALUE}, current:{}",
                 input.len()
-            )
+            ))
         );
+
         if !self.is_latin(input) {
-            return MetaString::new(input.to_string(), Encoding::Utf8, input.as_bytes().to_vec());
+            return Ok(Some(MetaString::new(
+                input.to_string(),
+                Encoding::Utf8,
+                input.as_bytes().to_vec(),
+                self.special_char1,
+                self.special_char2,
+            )?));
         }
-        let encoding = self.compute_encoding(input);
+
+        Ok(None)
+    }
+
+    pub fn encode(&self, input: &str) -> Result<MetaString, Error> {
+        if let Some(ms) = self._encode(input)? {
+            return Ok(ms);
+        }
+        let encoding = self.compute_encoding(input, None);
         self.encode_with_encoding(input, encoding)
     }
 
-    fn compute_encoding(&self, input: &str) -> Encoding {
+    pub fn encode_with_encodings(
+        &self,
+        input: &str,
+        encodings: &[Encoding],
+    ) -> Result<MetaString, Error> {
+        if let Some(ms) = self._encode(input)? {
+            return Ok(ms);
+        }
+        let encoding = self.compute_encoding(input, Some(encodings));
+        self.encode_with_encoding(input, encoding)
+    }
+
+    fn compute_encoding(&self, input: &str, encodings: Option<&[Encoding]>) -> Encoding {
+        let allow = |e: Encoding| encodings.map_or(true, |opts| opts.contains(&e));
         let statistics = self.compute_statistics(input);
-        if statistics.can_lower_special_encoded {
+        if statistics.can_lower_special_encoded && allow(Encoding::LowerSpecial) {
             return Encoding::LowerSpecial;
         }
         if statistics.can_lower_upper_digit_special_encoded {
-            if statistics.digit_count != 0 {
+            if statistics.digit_count != 0 && allow(Encoding::LowerUpperDigitSpecial) {
                 return Encoding::LowerUpperDigitSpecial;
             }
             let upper_count: usize = statistics.upper_count;
-            if upper_count == 1 && input.chars().next().unwrap().is_uppercase() {
+            if upper_count == 1
+                && input.chars().next().unwrap().is_uppercase()
+                && allow(Encoding::FirstToLowerSpecial)
+            {
                 return Encoding::FirstToLowerSpecial;
             }
-            if ((input.len() + upper_count) * 5) < (input.len() * 6) {
+            if ((input.len() + upper_count) * 5) < (input.len() * 6)
+                && allow(Encoding::AllToLowerSpecial)
+            {
                 return Encoding::AllToLowerSpecial;
             }
-            return Encoding::LowerUpperDigitSpecial;
+            if allow(Encoding::LowerUpperDigitSpecial) {
+                return Encoding::LowerUpperDigitSpecial;
+            }
         }
         Encoding::Utf8
     }
@@ -138,8 +234,7 @@ impl MetaStringEncoder {
                 && !(c.is_lowercase()
                     || c.is_uppercase()
                     || c.is_ascii_digit()
-                    || c == '.'
-                    || c == '_')
+                    || (c == self.special_char1 || c == self.special_char2))
             {
                 can_lower_upper_digit_special_encoded = false;
             }
@@ -169,45 +264,87 @@ impl MetaStringEncoder {
         encoding: Encoding,
     ) -> Result<MetaString, Error> {
         if input.is_empty() {
-            return MetaString::new(input.to_string(), Encoding::Utf8, vec![]);
+            return MetaString::new(
+                input.to_string(),
+                Encoding::Utf8,
+                vec![],
+                self.special_char1,
+                self.special_char2,
+            );
         }
         ensure!(
             input.len() < SHORT_MAX_VALUE,
-            anyhow!(
+            Error::encode_error(format!(
                 "Meta string is too long, max:{SHORT_MAX_VALUE}, current:{}",
                 input.len()
-            )
+            ))
         );
         ensure!(
             encoding == Encoding::Utf8 || self.is_latin(input),
-            anyhow!("Non-ASCII characters in meta string are not allowed")
+            Error::encode_error("Non-ASCII characters in meta string are not allowed")
         );
 
         if input.is_empty() {
-            return MetaString::new(input.to_string(), Encoding::Utf8, vec![]);
+            return MetaString::new(
+                input.to_string(),
+                Encoding::Utf8,
+                vec![],
+                self.special_char1,
+                self.special_char2,
+            );
         };
 
         match encoding {
             Encoding::LowerSpecial => {
                 let encoded_data = self.encode_lower_special(input)?;
-                MetaString::new(input.to_string(), encoding, encoded_data)
+                MetaString::new(
+                    input.to_string(),
+                    encoding,
+                    encoded_data,
+                    self.special_char1,
+                    self.special_char2,
+                )
             }
             Encoding::LowerUpperDigitSpecial => {
                 let encoded_data = self.encode_lower_upper_digit_special(input)?;
-                MetaString::new(input.to_string(), encoding, encoded_data)
+                MetaString::new(
+                    input.to_string(),
+                    encoding,
+                    encoded_data,
+                    self.special_char1,
+                    self.special_char2,
+                )
             }
             Encoding::FirstToLowerSpecial => {
                 let encoded_data = self.encode_first_to_lower_special(input)?;
-                MetaString::new(input.to_string(), encoding, encoded_data)
+                MetaString::new(
+                    input.to_string(),
+                    encoding,
+                    encoded_data,
+                    self.special_char1,
+                    self.special_char2,
+                )
             }
             Encoding::AllToLowerSpecial => {
                 let upper_count = input.chars().filter(|c| c.is_uppercase()).count();
                 let encoded_data = self.encode_all_to_lower_special(input, upper_count)?;
-                MetaString::new(input.to_string(), encoding, encoded_data)
+                MetaString::new(
+                    input.to_string(),
+                    encoding,
+                    encoded_data,
+                    self.special_char1,
+                    self.special_char2,
+                )
             }
             Encoding::Utf8 => {
                 let encoded_data = input.as_bytes().to_vec();
-                MetaString::new(input.to_string(), Encoding::Utf8, encoded_data)
+                MetaString::new(
+                    input.to_string(),
+                    Encoding::Utf8,
+                    encoded_data,
+                    self.special_char1,
+                    self.special_char2,
+                )
             }
         }
     }
@@ -242,9 +379,29 @@ impl MetaStringEncoder {
     }
 
     pub fn encode_first_to_lower_special(&self, input: &str) -> Result<Vec<u8>, Error> {
-        let mut chars: Vec<char> = input.chars().collect();
-        chars[0] = chars[0].to_lowercase().next().unwrap();
-        self.encode_generic(&chars.iter().collect::<String>(), 5)
+        if input.is_empty() {
+            return self.encode_generic("", 5);
+        }
+
+        let mut iter = input.char_indices();
+        let (first_idx, first_char) = iter.next().unwrap();
+
+        let lower = first_char.to_lowercase().to_string();
+
+        // Fast path: if lowercase has the same byte length and is ASCII,
+        // we can modify the first byte directly without rebuilding the string.
+        if lower.len() == first_char.len_utf8() && first_char.is_ascii() {
+            let mut bytes = input.as_bytes().to_owned();
+            bytes[first_idx] = lower.as_bytes()[0];
+            return self.encode_generic(std::str::from_utf8(&bytes).unwrap(), 5);
+        }
+
+        // rebuild only the necessary prefix + suffix (still efficient).
+        let (_, rest) = input.split_at(first_char.len_utf8());
+        let mut result = String::with_capacity(input.len() + lower.len() - first_char.len_utf8());
+        result.push_str(&lower);
+        result.push_str(rest);
+        self.encode_generic(&result, 5)
     }
 
     pub fn encode_all_to_lower_special(
@@ -272,23 +429,23 @@ impl MetaStringEncoder {
                 '_' => Ok(27),
                 '$' => Ok(28),
                 '|' => Ok(29),
-                _ => Err(anyhow!(
-                    "Unsupported character for LOWER_UPPER_DIGIT_SPECIAL encoding: {c}"
-                ))?,
+                _ => Err(Error::encode_error(format!(
+                    "Unsupported character for LOWER_UPPER_DIGIT_SPECIAL encoding: {c}",
+                )))?,
             },
             6 => match c {
                 'a'..='z' => Ok(c as u8 - b'a'),
                 'A'..='Z' => Ok(c as u8 - b'A' + 26),
                 '0'..='9' => Ok(c as u8 - b'0' + 52),
                 _ => {
-                    if c == '.' {
+                    if c == self.special_char1 {
                         Ok(62)
-                    } else if c == '_' {
+                    } else if c == self.special_char2 {
                         Ok(63)
                     } else {
-                        Err(anyhow!(
-                            "Invalid character value for LOWER_SPECIAL decoding: {c:?}"
-                        ))?
+                        Err(Error::encode_error(format!(
+                            "Invalid character value for LOWER_SPECIAL decoding: {c:?}",
+                        )))?
                     }
                 }
             },
@@ -298,21 +455,40 @@ impl MetaStringEncoder {
 }
 
 impl MetaStringDecoder {
-    pub fn new() -> Self {
-        MetaStringDecoder {}
+    pub const fn new(special_char1: char, special_char2: char) -> Self {
+        MetaStringDecoder {
+            special_char1,
+            special_char2,
+        }
     }
 
-    pub fn decode(&self, encoded_data: &[u8], encoding: Encoding) -> Result<String, Error> {
-        if encoded_data.is_empty() {
-            return Ok("".to_string());
-        }
-        match encoding {
-            Encoding::LowerSpecial => self.decode_lower_special(encoded_data),
-            Encoding::LowerUpperDigitSpecial => self.decode_lower_upper_digit_special(encoded_data),
-            Encoding::FirstToLowerSpecial => self.decode_rep_first_lower_special(encoded_data),
-            Encoding::AllToLowerSpecial => self.decode_rep_all_to_lower_special(encoded_data),
-            Encoding::Utf8 => Ok(String::from_utf8_lossy(encoded_data).into_owned()),
-        }
+    pub fn decode(&self, encoded_data: &[u8], encoding: Encoding) -> Result<MetaString, Error> {
+        let str = {
+            if encoded_data.is_empty() {
+                Ok("".to_string())
+            } else {
+                match encoding {
+                    Encoding::LowerSpecial => self.decode_lower_special(encoded_data),
+                    Encoding::LowerUpperDigitSpecial => {
+                        self.decode_lower_upper_digit_special(encoded_data)
+                    }
+                    Encoding::FirstToLowerSpecial => {
+                        self.decode_rep_first_lower_special(encoded_data)
+                    }
+                    Encoding::AllToLowerSpecial => {
+                        self.decode_rep_all_to_lower_special(encoded_data)
+                    }
+                    Encoding::Utf8 => Ok(String::from_utf8_lossy(encoded_data).into_owned()),
+                }
+            }
+        }?;
+        MetaString::new(
+            str,
+            encoding,
+            Vec::from(encoded_data),
+            self.special_char1,
+            self.special_char2,
+        )
     }
 
     fn decode_lower_special(&self, data: &[u8]) -> Result<String, Error> {
@@ -377,9 +553,9 @@ impl MetaStringDecoder {
             27 => Ok('_'),
             28 => Ok('$'),
             29 => Ok('|'),
-            _ => Err(anyhow!(
-                "Invalid character value for LOWER_SPECIAL decoding: {char_value}"
-            ))?,
+            _ => Err(Error::encode_error(format!(
+                "Invalid character value for LOWER_SPECIAL decoding: {char_value}",
+            )))?,
         }
     }
 
@@ -388,11 +564,11 @@ impl MetaStringDecoder {
             0..=25 => Ok((b'a' + char_value) as char),
             26..=51 => Ok((b'A' + char_value - 26) as char),
             52..=61 => Ok((b'0' + char_value - 52) as char),
-            62 => Ok('.'),
-            63 => Ok('_'),
-            _ => Err(anyhow!(
-                "Invalid character value for LOWER_UPPER_DIGIT_SPECIAL decoding: {char_value}"
-            ))?,
+            62 => Ok(self.special_char1),
+            63 => Ok(self.special_char2),
+            _ => Err(Error::encode_error(format!(
+                "Invalid character value for LOWER_UPPER_DIGIT_SPECIAL decoding: {char_value}",
+            )))?,
         }
     }
 
